@@ -23,6 +23,9 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: acctproc.c,v $
+** Revision 1.17  2004/12/14 15:05:00  gerlof
+** Implementation of patch-recognition for disk and network-statistics.
+**
 ** Revision 1.16  2004/06/01 11:57:22  gerlof
 ** Consider other standard accounting-files, i.e. /var/account/pacct.
 **
@@ -76,7 +79,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.16 2004/06/01 11:57:22 gerlof Exp $";
+static const char rcsid[] = "$Id: acctproc.c,v 1.17 2004/12/14 15:05:00 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -86,7 +89,6 @@ static const char rcsid[] = "$Id: acctproc.c,v 1.16 2004/06/01 11:57:22 gerlof E
 #include <errno.h>
 #include <unistd.h>
 #include <sys/time.h>
-#include <linux/acct.h>
 #include <sys/stat.h>
 #include <sys/ipc.h>
 #include <sys/sem.h>
@@ -101,9 +103,11 @@ static const char rcsid[] = "$Id: acctproc.c,v 1.16 2004/06/01 11:57:22 gerlof E
 
 static	char 	acctatop;	/* boolean: atop's own accounting busy ?  */
 static	off_t	acctsize;	/* previous size of account file	  */
+static	int	acctrecsz;	/* size of account record		  */
 int   		acctfd   = -1;	/* fd of account file       		  */
 
-static count_t 	acctexp(comp_t ct);
+static count_t 	acctexp (comp_t  ct);
+static count_t	acctexp2(comp2_t ct);
 
 struct pacctadm {
 	char		*name;
@@ -143,7 +147,7 @@ struct sembuf	semclaim = {0, -1, SEM_UNDO},
 int
 acctswon(void)
 {
-	int			i, j, semid;
+	int			i, j, semid, dev;
 	static ushort 		vals[] = {1, SEMTOTAL};
 	union {ushort *array;}	arg = {vals};
 	struct stat		statbuf;
@@ -192,16 +196,32 @@ acctswon(void)
 						return 0;
 					}
 
+					/*
+					** determine the acctrec size being
+					** used (patched kernel?)
+					*/
+					dev = statbuf.st_size -
+					      pacctadm[i].stat.st_size;
+
+					if (dev % sizeof(struct acct))
+					{
+					   acctrecsz = sizeof(struct acct_atop);
+					   kernpatch |= PATCHACCT;
+					}
+					else
+					{
+					   acctrecsz = sizeof(struct acct);
+					}
+
 					/*
 					** accounting successfully switched on
 					*/
-					acctsize = acctprocnt() *
-							sizeof(struct acct);
+					acctsize = acctprocnt() * acctrecsz;
 	
  					/*
 					** reposition to actual file-size
 					*/
-					(void) lseek(acctfd, acctsize,SEEK_SET);
+					(void)lseek(acctfd, acctsize, SEEK_SET);
 					return 1;
 				}
 			}
@@ -327,8 +347,44 @@ acctswon(void)
 
 	acctatop = 1;
 
-	acctsize = acctprocnt() * sizeof(struct acct);
+	/*
+	** determine if patch for extended accounting is installed
+	*/ 
+	(void) fstat(acctfd, &statbuf);
 
+	if (statbuf.st_size == 0)	/* no acct record written yet */
+	{
+		/*
+		** let's write one record
+		*/
+		if ( fork() == 0 )
+			exit(0);	/* let the child finish */
+
+		(void) wait((int *) 0); /* let the parent wait  */
+
+		(void) fstat(acctfd, &statbuf);
+
+		if (statbuf.st_size % sizeof(struct acct     ) == 0  &&
+		    statbuf.st_size % sizeof(struct acct_atop) == 0    )
+		{
+			if ( fork() == 0 )
+				exit(0);	/* let the child finish */
+
+			(void) wait((int *) 0); /* let the parent wait  */
+		}
+	}
+
+	if (statbuf.st_size % sizeof(struct acct))
+	{
+	 	acctrecsz = sizeof(struct acct_atop);
+		kernpatch |= PATCHACCT;
+	}
+	else
+	{
+		acctrecsz = sizeof(struct acct);
+	}
+
+	acctsize = acctprocnt() * acctrecsz;
 
  	/*
 	** reposition to actual file size
@@ -400,7 +456,7 @@ acctswoff(void)
 int
 acctprocnt(void)
 {
-	struct stat 	statacc;
+	struct stat 		statacc;
 
 	/*
 	** if accounting not supported, skip call
@@ -422,7 +478,7 @@ acctprocnt(void)
 		acctsize = 0;
 	}
 
-	return( (statacc.st_size - acctsize) / sizeof(struct acct) );
+	return (statacc.st_size - acctsize) / acctrecsz;
 }
 
 /*
@@ -434,6 +490,7 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 {
 	register int 		nrexit;
 	register struct pstat 	*api;
+	struct acct_atop	acctrec_atop;
 	struct acct 		acctrec;
 
 	/*
@@ -447,34 +504,75 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 	*/
 	for  (nrexit=0, api=accproc; nrexit < nrprocs; nrexit++, api++)
 	{
-		if ( read(acctfd, &acctrec, sizeof(acctrec)) < sizeof(acctrec) )
-			break;		/* unexpected end of account file */
+		if ( !(kernpatch & PATCHACCT) )
+		{
+			if ( read(acctfd, &acctrec, acctrecsz) < acctrecsz )
+				break;	/* unexpected end of account file */
 
-		/*
-		** fill process info from accounting-record
-		*/
-		api->gen.state  = 'E';
-		api->gen.excode = acctrec.ac_exitcode;
-		api->gen.ruid   = acctrec.ac_uid;
-		api->gen.rgid   = acctrec.ac_gid;
-		api->gen.btime  = acctrec.ac_btime;
-		api->cpu.stime  = acctexp(acctrec.ac_stime);
-		api->cpu.utime  = acctexp(acctrec.ac_utime);
-		api->mem.minflt = acctexp(acctrec.ac_minflt);
-		api->mem.majflt = acctexp(acctrec.ac_majflt);
-		api->mem.iosw   = acctexp(acctrec.ac_swaps);
-		api->dsk.ior    = acctexp(acctrec.ac_rw);
+			/*
+			** fill process info from accounting-record
+			*/
+			api->gen.state  = 'E';
+			api->gen.excode = acctrec.ac_exitcode;
+			api->gen.ruid   = acctrec.ac_uid;
+			api->gen.rgid   = acctrec.ac_gid;
+			api->gen.btime  = acctrec.ac_btime;
+			api->cpu.stime  = acctexp(acctrec.ac_stime);
+			api->cpu.utime  = acctexp(acctrec.ac_utime);
+			api->mem.minflt = acctexp(acctrec.ac_minflt);
+			api->mem.majflt = acctexp(acctrec.ac_majflt);
+			api->mem.iosw   = acctexp(acctrec.ac_swaps);
+			api->dsk.rio    = acctexp(acctrec.ac_rw);
 
-		strcpy(api->gen.name, acctrec.ac_comm);
+			strcpy(api->gen.name, acctrec.ac_comm);
+		}
+		else
+		{
+			if ( read(acctfd, &acctrec_atop, acctrecsz) <acctrecsz)
+				break;	/* unexpected end of account file */
+
+			/*
+			** fill process info from accounting-record
+			*/
+			api->gen.state  = 'E';
+			api->gen.pid    = acctrec_atop.ac_pid;
+			api->gen.excode = acctrec_atop.ac_exitcode;
+			api->gen.ruid   = acctrec_atop.ac_uid;
+			api->gen.rgid   = acctrec_atop.ac_gid;
+			api->gen.btime  = acctrec_atop.ac_btime;
+			api->cpu.stime  = acctexp (acctrec_atop.ac_stime);
+			api->cpu.utime  = acctexp (acctrec_atop.ac_utime);
+			api->mem.minflt = acctexp (acctrec_atop.ac_minflt);
+			api->mem.majflt = acctexp (acctrec_atop.ac_majflt);
+			api->mem.iosw   = acctexp (acctrec_atop.ac_swaps);
+			api->mem.vmem   = acctexp (acctrec_atop.ac_mem);
+			api->mem.rmem   = acctexp (acctrec_atop.ac_rss);
+			api->dsk.rio    = acctexp (acctrec_atop.ac_bread);
+			api->dsk.wio    = acctexp (acctrec_atop.ac_bwrite);
+			api->dsk.rsz    = acctexp2(acctrec_atop.ac_dskrsz);
+			api->dsk.wsz    = acctexp2(acctrec_atop.ac_dskwsz);
+			api->net.tcpsnd = acctexp (acctrec_atop.ac_tcpsnd);
+			api->net.tcprcv = acctexp (acctrec_atop.ac_tcprcv);
+			api->net.tcpssz = acctexp2(acctrec_atop.ac_tcpssz);
+			api->net.tcprsz = acctexp2(acctrec_atop.ac_tcprsz);
+			api->net.udpsnd = acctexp (acctrec_atop.ac_udpsnd);
+			api->net.udprcv = acctexp (acctrec_atop.ac_udprcv);
+			api->net.udpssz = acctexp2(acctrec_atop.ac_udpssz);
+			api->net.udprsz = acctexp2(acctrec_atop.ac_udprsz);
+			api->net.rawsnd = acctexp (acctrec_atop.ac_rawsnd);
+			api->net.rawrcv = acctexp (acctrec_atop.ac_rawrcv);
+
+			strcpy(api->gen.name, acctrec_atop.ac_comm);
+		}
 	}
 
-	acctsize += (nrexit * sizeof(acctrec));
+	acctsize += nrexit * acctrecsz;
 
 	return nrexit;
 }
 
 /*
-** expand the special compression-method
+** expand the special compression-methods
 */
 static count_t
 acctexp(comp_t ct)
@@ -482,11 +580,26 @@ acctexp(comp_t ct)
         count_t	exp;
         count_t val;
 
-        exp = (ct >> 13) & 0x7;		/* obtain base-8 exponent */
-        val =  ct & 0x1fff;		/* obtain fraction        */
+        exp = (ct >> 13) & 0x7;		/* obtain  3 bits base-8 exponent */
+        val =  ct & 0x1fff;		/* obtain 13 bits mantissa        */
 
         while (exp-- > 0)
                 val <<= 3;
 
         return val;
 }
+
+static count_t
+acctexp2(comp2_t ct)
+{
+	count_t exp;
+	count_t val;
+
+	exp = (ct >> 20) & 0xfff;         /* obtain  5 bits base-2 exponent */
+	val =  ct & 0xfffff;              /* obtain 20 bits mantissa        */
+
+	while (exp-- > 0)
+		val <<= 2;
+
+	return val;
+}
