@@ -109,6 +109,17 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: atop.c,v $
+** Revision 1.20  2004/12/14 15:05:38  gerlof
+** Implementation of patch-recognition for disk and network-statistics.
+**
+** Revision 1.19  2004/10/26 13:42:49  gerlof
+** Also lock current physical pages in memory.
+**
+** Revision 1.18  2004/09/15 08:23:42  gerlof
+** Set resource limit for locked memory to infinite, because
+** in certain environments it is set to 32K (causes atop-malloc's
+** to fail).
+**
 ** Revision 1.17  2004/05/06 09:45:44  gerlof
 ** Ported to kernel-version 2.6.
 **
@@ -162,7 +173,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: atop.c,v 1.17 2004/05/06 09:45:44 gerlof Exp $";
+static const char rcsid[] = "$Id: atop.c,v 1.20 2004/12/14 15:05:38 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -177,6 +188,8 @@ static const char rcsid[] = "$Id: atop.c,v 1.17 2004/05/06 09:45:44 gerlof Exp $
 #include <signal.h>
 #include <sys/utsname.h>
 #include <string.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 
 #include "atop.h"
 #include "acctproc.h"
@@ -187,6 +200,11 @@ static const char rcsid[] = "$Id: atop.c,v 1.17 2004/05/06 09:45:44 gerlof Exp $
 #define	PROCCHUNK	100	/* portion-size for process-table expansion */
 #define	MAXFL		64      /* maximum number of command-line flags     */
 
+/*
+** installed kernel patches
+*/
+char		kernpatch;
+
 /*
 ** timing info
 */
@@ -260,6 +278,7 @@ main(int argc, char *argv[])
 	register int	i;
 	int		c;
 	char		*p;
+	struct rlimit	rlim;
 
 	/*
 	** check if defaults-file $HOME/.atoprc present;
@@ -418,11 +437,19 @@ main(int argc, char *argv[])
 	/*
 	** lock ATOP in memory to get reliable samples (also when
 	** memory is low);
+	** ignored if not running under superuser priviliges!
+	*/
+	rlim.rlim_cur	= RLIM_INFINITY;
+	rlim.rlim_max	= RLIM_INFINITY;
+	(void) setrlimit(RLIMIT_MEMLOCK, &rlim);
+
+	(void) mlockall(MCL_CURRENT|MCL_FUTURE);
+
+	/*
 	** increment CPU scheduling-priority to get reliable samples (also
 	** during heavy CPU load);
 	** ignored if not running under superuser priviliges!
 	*/
-	(void) mlockall(MCL_FUTURE);
 	(void) nice(-20);
 
 	/*
