@@ -23,6 +23,12 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: photosyst.c,v $
+** Revision 1.19  2004/10/28 08:31:23  gerlof
+** New counter: vm committed space
+**
+** Revision 1.18  2004/09/24 10:02:35  gerlof
+** Wrong cpu-numbers for system level statistics.
+**
 ** Revision 1.17  2004/05/07 05:27:37  gerlof
 ** Recognize new disk-names and support of diskname-modification.
 **
@@ -77,7 +83,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.17 2004/05/07 05:27:37 gerlof Exp $";
+static const char rcsid[] = "$Id: photosyst.c,v 1.19 2004/10/28 08:31:23 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -172,6 +178,7 @@ photosyst(struct sstat *si)
 			{
 				i = atoi(&nam[3]);
 
+				si->cpu.cpu[i].cpunr	= i;
 				si->cpu.cpu[i].utime	= cnts[0];
 				si->cpu.cpu[i].ntime	= cnts[1];
 				si->cpu.cpu[i].stime	= cnts[2];
@@ -266,10 +273,11 @@ photosyst(struct sstat *si)
 	si->mem.slabmem		= (count_t) 0;
 	si->mem.totswap  	= (count_t)-1;
 	si->mem.freeswap 	= (count_t)-1;
+	si->mem.committed 	= (count_t) 0;
 
 	if ( (fp = fopen("meminfo", "r")) != NULL)
 	{
-		int	nrfields = 7;	/* number of fields to be filled */
+		int	nrfields = 8;	/* number of fields to be filled */
 
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL && 
 								nrfields > 0)
@@ -357,6 +365,11 @@ photosyst(struct sstat *si)
 					si->mem.slabmem = cnts[0]*1024/pgsz;
 					nrfields--;
 				}
+			else	if (strcmp("Committed_AS:", nam) == EQ)
+				{
+					si->mem.committed = cnts[0]*1024/pgsz;
+					nrfields--;
+				}
 		}
 
 		fclose(fp);
