@@ -34,12 +34,16 @@ struct	memstat {
 	count_t	buffermem;	/* number of buffer   pages	*/
 	count_t	slabmem;	/* number of slab     pages	*/
 	count_t	cachemem;	/* number of cache    pages	*/
+	count_t	committed;	/* number of reserved pages	*/
 
 	count_t	swouts;		/* number of pages swapped out	*/
 	count_t	swins;		/* number of pages swapped in	*/
 
 	count_t	totswap;	/* number of pages in swap	*/
 	count_t	freeswap;	/* number of free swap pages	*/
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
+	count_t	future3;	/* reserved for future use	*/
 };
 
 /************************************************************************/
@@ -55,11 +59,15 @@ struct	netstat {
 	count_t	ipout;	/* IP  transmitted packets		*/
 	count_t	ipindel;/* IP  locally delivered packets	*/
 	count_t	ipfrw;	/* IP  forwarded packets		*/
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
+	count_t	future3;	/* reserved for future use	*/
 };
 
 /************************************************************************/
 
 struct percpu {
+	int	cpunr;
 	count_t	stime;	/* system  time in clock ticks		*/
 	count_t	utime;	/* user    time in clock ticks		*/
 	count_t	ntime;	/* nice    time in clock ticks		*/
@@ -67,12 +75,17 @@ struct percpu {
 	count_t	wtime;	/* iowait  time in clock ticks		*/
 	count_t	Itime;	/* irq     time in clock ticks		*/
 	count_t	Stime;	/* softirq time in clock ticks		*/
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
+	count_t	future3;	/* reserved for future use	*/
 };
 
 struct	cpustat {
 	count_t	devint;	/* number of device interrupts 		*/
 	count_t	csw;	/* number of context switches		*/
 	count_t	nrcpu;	/* number of cpu's 			*/
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
 
 	struct percpu   all;
 	struct percpu   cpu[MAXCPU];
@@ -88,6 +101,9 @@ struct	perxdsk {
         count_t	nwblk;	/* number of sectors written            */
         count_t	io_ms;	/* number of millisecs spent for I/O    */
         count_t	avque;	/* average queue length                 */
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
+	count_t	future3;	/* reserved for future use	*/
 };
 
 struct xdskstat {
@@ -103,6 +119,9 @@ struct	perintf {
         count_t	rpack;	/* number of read packets               */
         count_t	sbyte;	/* number of written bytes              */
         count_t	spack;	/* number of written packets            */
+	count_t	future1;	/* reserved for future use	*/
+	count_t	future2;	/* reserved for future use	*/
+	count_t	future3;	/* reserved for future use	*/
 };
 
 struct intfstat {
