@@ -23,6 +23,18 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: showgeneric.c,v $
+** Revision 1.35  2004/12/14 15:06:41  gerlof
+** Implementation of patch-recognition for disk and network-statistics.
+**
+** Revision 1.34  2004/09/27 11:01:13  gerlof
+** Corrected usage-info as suggested by Edelhard Becker.
+**
+** Revision 1.33  2004/09/13 09:19:14  gerlof
+** Modify subcommands (former 's' -> 'v', 'v' -> 'V', new 's').
+**
+** Revision 1.32  2004/08/31 09:52:47  root
+** information about underlying threads.
+**
 ** Revision 1.31  2004/06/01 11:57:58  gerlof
 ** Regular expressions for selections on process-name and user-name.
 **
@@ -119,7 +131,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showgeneric.c,v 1.31 2004/06/01 11:57:58 gerlof Exp $";
+static const char rcsid[] = "$Id: showgeneric.c,v 1.35 2004/12/14 15:06:41 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -150,7 +162,6 @@ static const char rcsid[] = "$Id: showgeneric.c,v 1.31 2004/06/01 11:57:58 gerlo
 extern unsigned long	interval;
 extern int		deviatonly;
 extern char		rawreadflag;
-extern char		kernpatch;
 extern struct utsname	utsname;
 extern char		flaglist[];
 
@@ -233,8 +244,29 @@ generic_samp(time_t curtime, int nsecs,
 				sortorder = "MEM";
 				break;
 
+			   case 's':
+				showtype  = 's';
+				curcompar = compcpu;
+				sortorder = "CPU";
+				break;
+
+			   case 'd':
+				if ( !(kernpatch & PATCHSTAT) )
+				{
+					fprintf(stderr,
+						"No kernel-patch installed "
+						"(no disk-statistics)\n");
+					sleep(3);
+					break;
+				}
+
+				showtype  = 'd';
+				curcompar = compdsk;
+				sortorder = "DSK";
+				break;
+
 			   case 'n':
-				if (!kernpatch)
+				if ( !(kernpatch & PATCHSTAT) )
 				{
 					fprintf(stderr,
 						"No kernel-patch installed "
@@ -248,8 +280,8 @@ generic_samp(time_t curtime, int nsecs,
 				sortorder = "NET";
 				break;
 
-			   case 's':
-				showtype  = 's';
+			   case 'v':
+				showtype  = 'v';
 				break;
 
 			   case 'c':
@@ -269,7 +301,7 @@ generic_samp(time_t curtime, int nsecs,
 				maxintlines = 3;
 				break;
 
-			   case 'v':
+			   case 'V':
 				printf("%s\n", getversion());
 				cleanstop(0);
 
@@ -324,7 +356,7 @@ generic_samp(time_t curtime, int nsecs,
 	** compute the total capacity of this system for the 
 	** four main resources
 	*/
-	totalcap(&syscap, sstat);
+	totalcap(&syscap, sstat, pstat, nact);
 
         /*
         ** prepare screen or file output for new sample
@@ -577,11 +609,27 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** disk-specific figures per process
+			   */
+			   case 'd':
+				if ( !(kernpatch & PATCHSTAT) )
+				{
+					printw("No kernel-patch installed; "
+					       "request ignored!");
+					break;
+				}
+				showtype  = 'd';
+				curcompar = compdsk;
+				sortorder = "DSK";
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** network-specific figures per process
 			   */
 			   case 'n':
-				if (!kernpatch)
+				if ( !(kernpatch & PATCHSTAT) )
 				{
 					printw("No kernel-patch installed; "
 					       "request ignored!");
@@ -594,10 +642,10 @@ generic_samp(time_t curtime, int nsecs,
 				break;
 
 			   /*
-			   ** static info per process
+			   ** various info per process
 			   */
-			   case 's':
-				showtype  = 's';
+			   case 'v':
+				showtype  = 'v';
 				firstproc = 0;
 				break;
 
@@ -609,6 +657,16 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** scheduling-values per process
+			   */
+			   case 's':
+				showtype  = 's';
+				curcompar = compcpu;
+				sortorder = "CPU";
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** help wanted?
 			   */
@@ -876,7 +934,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** show version info
 			   */
-			   case 'v':
+			   case 'V':
 				printw("%s", getversion());
 				break;
 
@@ -982,16 +1040,18 @@ printg(const char *format, ...)
 static char *helptext[] = {
 	"Figures shown for active processes:",
 	"\t'g' - generic info (default)",
-	"\t'm' - memory-details",
-	"\t'n' - network-details",
-	"\t's' - static process-info",
+	"\t'm' - memory details",
+	"\t'd' - disk details",
+	"\t'n' - network details",
+	"\t's' - scheduling and thread-group info",
+	"\t'v' - various info (user/group, date/time)",
 	"\t'c' - full command-line per process",
 	"",
 	"Sort list of active processes by:",
-	"\t'C' - cpu-activity (default)",
-	"\t'M' - memory-consumption",
-	"\t'D' - disk-activity",
-	"\t'N' - network-activity",
+	"\t'C' - cpu activity (default)",
+	"\t'M' - memory consumption",
+	"\t'D' - disk activity",
+	"\t'N' - network activity",
 	"",
 	"Screen-handling:",
 	"\t^F  - show next     page in the process-list (forward)",
@@ -1011,7 +1071,7 @@ static char *helptext[] = {
 	"",
 	"\t'l' - limited lines for per-cpu, disk and interface resources",
 	"",
-	"\t'v' - version-information",
+	"\t'V' - version-information",
 	"\t'h' - help-information",
 	"\t'?' - help-information",
 	"\t'q' - quit this program",
@@ -1096,8 +1156,10 @@ generic_usage(void)
 {
 	printf("\t  -g  show general process-info (default)\n");
 	printf("\t  -m  show memory-related process-info\n");
+	printf("\t  -d  show disk-related process-info\n");
 	printf("\t  -n  show network-related process-info\n");
-	printf("\t  -s  show static process-info\n");
+	printf("\t  -s  show scheduling-related process-info\n");
+	printf("\t  -v  show various process-info (user/group, date/time)\n");
 	printf("\t  -c  show command-line per process\n");
 	printf("\t  -C  sort by cpu-consumption (default)\n");
 	printf("\t  -M  sort by memory-consumption\n");
@@ -1250,14 +1312,26 @@ do_flags(char *val)
 			sortorder = "MEM";
 			break;
 
+		   case 'd':
+			showtype  = 'd';
+			curcompar = compdsk;
+			sortorder = "DSK";
+			break;
+
 		   case 'n':
 			showtype  = 'n';
 			curcompar = compnet;
 			sortorder = "NET";
 			break;
 
+		   case 'v':
+			showtype  = 'v';
+			break;
+
 		   case 's':
 			showtype  = 's';
+			curcompar = compcpu;
+			sortorder = "CPU";
 			break;
 
 		   case 'c':
