@@ -22,6 +22,30 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: showlinux.c,v $
+** Revision 1.24  2004/12/14 15:06:48  gerlof
+** Implementation of patch-recognition for disk and network-statistics.
+**
+** Revision 1.23  2004/10/28 08:31:41  gerlof
+** New counter: vm committed space
+**
+** Revision 1.22  2004/09/24 10:02:46  gerlof
+** Wrong cpu-numbers for system level statistics.
+**
+** Revision 1.21  2004/09/23 08:21:10  gerlof
+** Added wait-percentage per cpu.
+**
+** Revision 1.20  2004/09/23 07:37:34  gerlof
+** Consistent handling of CPU percentages on system-level and process-level.
+**
+** Revision 1.19  2004/09/13 09:20:21  gerlof
+** Modify subcommands (former 's' -> 'v', 'v' -> 'V', new 's').
+**
+** Revision 1.18  2004/09/02 10:55:21  root
+** Added sleep-average to process-info.
+**
+** Revision 1.17  2004/08/31 09:53:31  gerlof
+** Show information about underlying threads.
+**
 ** Revision 1.16  2004/06/01 11:58:34  gerlof
 ** Regular expressions for selections on process-name and user-name.
 **
@@ -78,7 +102,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.16 2004/06/01 11:58:34 gerlof Exp $";
+static const char rcsid[] = "$Id: showlinux.c,v 1.24 2004/12/14 15:06:48 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -104,7 +128,6 @@ static const char rcsid[] = "$Id: showlinux.c,v 1.16 2004/06/01 11:58:34 gerlof
 #include "showgeneric.h"
 
 extern int      acctfd;
-extern char     kernpatch;
 extern time_t   pretime;        /* previous sample time          */
 
 extern int      osrel;
@@ -113,61 +136,75 @@ extern int      osvers;
 /*
 ** format-lines for printing
 */
-char 	*prcline   	= "PRC | sys %s | user %s | #procs %s | #exits %s |"
+char 	*prcline   	= "PRC | sys %s | user %s | #thr %s | #exits %s |"
 	       	     	  " #zombie %s |\n";
-
-
-char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW  USERNAME  PRI"
+/***************************************************************************/
+char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW USERNAME NTHRD"
 		     	  " STATUS S  %s CMD  %4d/%-4d\n";
-char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW  READ WRITE "
-		     	  "RNET SNET S  %s CMD  %4d/%-4d\n";
-
-char 	*genproclina1	= "%5d %s %s %s %s  %-9.9s %3d %-6s %c %3.0lf%% "
+char 	*genproclina1	= "%5d %s %s %s %s %-8.8s %3d/%c %-6s %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*genproclina2	= "%5d %s %s %s %s %s %s %s %s %c %3.0lf%% %.14s\n";
-
-char 	*genprocline1	= "%s %s                %-9.9s     %s"
+char 	*genprocline1	= "%s %s               %-8.8s       %s"
 			  " E %3.0lf%% <%.12s>\n";
+
+char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW RDDSK WRDSK "
+		     	  "RNET SNET S  %s CMD  %4d/%-4d\n";
+char 	*genproclina2	= "%5d %s %s %s %s %s %s %s %s %c %3.0lf%% %.14s\n";
 char 	*genprocline2	= "%s %s               (r&w %s)        "
 			  "  E %3.0lf%% <%.12s>\n";
-
-
+char 	*genprocline2p	= "%s %s %s %s %s %s %s %s E %3.0lf%% <%.12s>\n";
+/***************************************************************************/
 char	*memprochdr	= "\n  PID MINFLT MAJFLT #SWP VSTEXT "
 		     	  " VSIZE  RSIZE  VGROW  RGROW  %s CMD  %4d/%-4d\n";
 char 	*memproclina	= "%5d %s %s %s %s %s %s %s %s %3.0lf%% "
 	                  "%.14s\n";
 char 	*memprocline	= "%s %s %s                                  "
 			  "  %3.0lf%% <%.12s>\n";
-
-
-char	*netprochdr	= "\n  PID   TCPRCV  TCPSND    UDPRCV  UDPSND"
-		     	  "                     %s CMD  %4d/%-4d\n";
-char 	*netproclina	= "%5d  %s %s   %s %s                    %3.0lf%% "
-	                  "%.14s\n";
+char 	*memproclinep	= "%5d %s %s %s        %s %s %s %s"
+			  " %3.0lf%% <%.12s>\n";
+/***************************************************************************/
+char	*dskprochdr	= "\n  PID  READDISK AVGSIZE TOTSIZE   WRITEDISK "
+			  "AVGSIZE TOTSIZE  %s CMD  %4d/%-4d\n";
+char 	*dskproclina	= "%5d %s %s  %s   %s %s  %s %3.0lf%% %.14s\n";
+char 	*dskprocline	= " (reads&writes              %s)   "
+			  "             %3.0lf%% <%.12s>\n";
+char 	*dskproclinep	= "%5d %s %s  %s   %s %s  %s %3.0lf%% <%.12s>\n";
+/***************************************************************************/
+char	*netprochdr	= "\n  PID TCPRCV/AVSZ TCPSND/AVSZ UDPRCV/AVSZ"
+			  " UDPSND/AVSZ RAWRS  %s CMD  %4d/%-4d\n";
+char 	*netproclina	= "%5d %s/%s %s/%s %s/%s %s/%s %2llu %2llu "
+			  "%3.0lf%% %.14s\n";
 char 	*netprocline	= "                                        "
 			  "              %3.0lf%% <%.12s>\n";
-
-
-char	*staprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
-	                  " STATUS S  %s CMD  %4d/%-4d\n";
-char 	*staproclina	= "%5d  %-8.8s %-8.8s  %s %s      %s    %c %3.0lf%% "
+char 	*netproclinep	= "%5d %s/%s %s/%s %s/%s %s/%s %2llu %2llu "
+			  "%3.0lf%% <%.12s>\n";
+/***************************************************************************/
+char	*varprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
+	                  "STATUS  S  %s CMD  %4d/%-4d\n";
+char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %-6s  %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*staprocline	= " %-8.8s %-8.8s  %s %s      %s E %3.0lf%% <%.12s>\n";
-
-
+char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %s  E %3.0lf%% <%.12s>\n";
+/***************************************************************************/
 char	*cmdprochdr	= "\n  PID  %s COMMAND-LINE                           "
 			  "                    %4d/%-4d\n";
 char 	*cmdproclina	= "%5d %3.0lf%% %s\n";
 char 	*cmdprocline	= "%3.0lf%% <%.66s>\n";
+/***************************************************************************/
+char	*schprochdr	= "\n  PID  TGID  NTHRD POLICY NICE PRIO RTPRIO "
+			  "CURCPU STATUS  S  %s CMD  %4d/%-4d\n";
+char 	*schproclina	= "%5d %5d %4d/%c %-6s %4d %4d %6d %6d %-6s "
+			  " %c %3.0lf%% %.14s\n";
+char 	*schprocline	= "                                             "
+			  "%s  E %3.0lf%% <%.12s>\n";
+/***************************************************************************/
 
 /*
 ** calculate the total consumption on system-level for the 
 ** four main resources
 */
 void
-totalcap(struct syscap *psc, struct sstat *sstat)
+totalcap(struct syscap *psc, struct sstat *sstat, struct pstat *pstat, int nact)
 {
-	int	i;
+	register int	i;
 	count_t	pgsz = getpagesz();
 
 	psc->availcpu	= sstat->cpu.all.stime +
@@ -177,23 +214,29 @@ totalcap(struct syscap *psc, struct sstat *sstat)
 			  sstat->cpu.all.wtime +
 			  sstat->cpu.all.Itime +
 			  sstat->cpu.all.Stime;
+	psc->availcpu  /= sstat->cpu.nrcpu;
 
 	psc->availmem	= sstat->mem.physmem * pgsz/1024;
 
-	psc->availnet	= sstat->net.tcpin  +
-			  sstat->net.tcpout +
-	                  sstat->net.udpin  +
-			  sstat->net.udpout;
+	if (kernpatch & PATCHSTAT)
+	{
+		/*
+		** calculate total number of accesses which have been
+		** issued by the active processes for disk and for network
+		*/
+		for (psc->availnet=psc->availdsk=0, i=0; i < nact; i++) 
+		{
+			psc->availnet += (pstat+i)->net.tcpsnd;
+			psc->availnet += (pstat+i)->net.tcprcv;
+			psc->availnet += (pstat+i)->net.udpsnd;
+			psc->availnet += (pstat+i)->net.udprcv;
+			psc->availnet += (pstat+i)->net.rawsnd;
+			psc->availnet += (pstat+i)->net.rawrcv;
 
-	/*
-	** calculate total number of accesses which have been
-	** issued by the active disks
-	*/
-	psc->availdsk = 0;
-
-	for (i=0; sstat->xdsk.xdsk[i].name[0]; i++)
-		psc->availdsk += sstat->xdsk.xdsk[i].nread +
-				 sstat->xdsk.xdsk[i].nwrite;
+			psc->availdsk += (pstat+i)->dsk.rio;
+			psc->availdsk += (pstat+i)->dsk.wio;
+		}
+	}
 }
 
 /*
@@ -215,7 +258,7 @@ pricumproc(struct pstat *pstat, int nact, int nproc, int nzomb, int nexit)
 
 	printg(prcline, val2cpustr(totst * 1000/HZ, format1),
 			val2cpustr(totut * 1000/HZ, format2),
-			val2valstr(nproc,           format3, 5),
+			val2valstr(nproc,           format3, 7),
 			acctfd >= 0 ?
 			          val2valstr(nexit, format4, 5) : "    ?",
 			val2valstr(nzomb,           format5, 4));
@@ -240,7 +283,7 @@ showgenproc(struct pstat *curstat, double perc)
 		/*
 		** show process-info of active process
 		*/
-		if (!kernpatch)
+		if ( !(kernpatch & PATCHSTAT) )
 		{
 			/*
 			** determine the user-name of process
@@ -254,12 +297,13 @@ showgenproc(struct pstat *curstat, double perc)
 			   curstat->gen.pid,
 			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			   val2memstr(curstat->mem.vgrow*1024.0,  format3,
+			   val2memstr(curstat->mem.vgrow*1024,    format3,
 								  KBFORMAT),
-			   val2memstr(curstat->mem.rgrow*1024.0,  format4,
+			   val2memstr(curstat->mem.rgrow*1024,    format4,
 								  KBFORMAT),
 			   username,
-			   curstat->cpu.pri,
+			   curstat->gen.nthr,
+			   curstat->gen.pid == curstat->gen.tgid ? 'L' : 'M',
 			   curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
 			   curstat->gen.state,
 			   perc, curstat->gen.name);
@@ -270,16 +314,18 @@ showgenproc(struct pstat *curstat, double perc)
 			   curstat->gen.pid,
 			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			   val2memstr(curstat->mem.vgrow*1024.0,  format3,
+			   val2memstr(curstat->mem.vgrow*1024,    format3,
 								  KBFORMAT),
-			   val2memstr(curstat->mem.rgrow*1024.0,  format4,
+			   val2memstr(curstat->mem.rgrow*1024,    format4,
 								  KBFORMAT),
-			   val2valstr(curstat->dsk.ior,           format5, 5),
-			   val2valstr(curstat->dsk.iow,           format6, 5),
-			   val2valstr(curstat->net.tcprcv+curstat->net.udprcv,
-			                                          format7, 4),
-			   val2valstr(curstat->net.tcpsnd+curstat->net.udpsnd,
-			                                          format8, 4),
+			   val2valstr(curstat->dsk.rio,           format5, 5),
+			   val2valstr(curstat->dsk.wio,           format6, 5),
+			   val2valstr(curstat->net.tcprcv +
+			              curstat->net.udprcv +
+			              curstat->net.rawrcv,        format7, 4),
+			   val2valstr(curstat->net.tcpsnd +
+			              curstat->net.udpsnd +
+			              curstat->net.rawsnd,        format8, 4),
 			   curstat->gen.state,
 			   perc, curstat->gen.name);
 		}
@@ -294,7 +340,7 @@ showgenproc(struct pstat *curstat, double perc)
 		else
 			printg("%5d ", curstat->gen.pid);
 
-		if (!kernpatch)
+		if ( !(kernpatch & PATCHSTAT) )
 		{
 			if ( (pwd = getpwuid(curstat->gen.ruid)) == NULL)
 				username = "unknown";
@@ -318,11 +364,33 @@ showgenproc(struct pstat *curstat, double perc)
 		}
 		else
 		{
+		   if (kernpatch & PATCHACCT)
+		   {
+			printg(genprocline2p,
+			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
+			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
+			   val2memstr(curstat->mem.vgrow*1024,    format3,
+								  KBFORMAT),
+			   val2memstr(curstat->mem.rgrow*1024,    format4,
+								  KBFORMAT),
+			   val2valstr(curstat->dsk.rio,           format5, 5),
+			   val2valstr(curstat->dsk.wio,           format6, 5),
+			   val2valstr(curstat->net.tcprcv +
+			              curstat->net.udprcv +
+			              curstat->net.rawrcv,        format7, 4),
+			   val2valstr(curstat->net.tcpsnd +
+			              curstat->net.udpsnd +
+			              curstat->net.rawsnd,        format8, 4),
+			   perc, curstat->gen.name);
+		   }
+		   else
+		   {
 			printg(genprocline2,
 			  val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			  val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			  val2valstr(curstat->dsk.ior, format3, 6),
+			  val2valstr(curstat->dsk.rio, format3, 6),
 			  perc, curstat->gen.name);
+		   }
 		}
 	}
 }
@@ -345,14 +413,14 @@ showmemproc(struct pstat *curstat, double perc)
 		*/
 		printg(memproclina,
 		  curstat->gen.pid,
-		  val2valstr(curstat->mem.minflt,        format1, 6),
-		  val2valstr(curstat->mem.majflt,        format2, 6),
-		  val2valstr(curstat->mem.iosw,          format3, 4),
-		  val2memstr(curstat->mem.shtext*1024.0, format4, KBFORMAT),
-		  val2memstr(curstat->mem.vmem  *1024.0, format5, KBFORMAT),
-		  val2memstr(curstat->mem.rmem  *1024.0, format6, KBFORMAT),
-		  val2memstr(curstat->mem.vgrow *1024.0, format7, KBFORMAT),
-		  val2memstr(curstat->mem.rgrow *1024.0, format8, KBFORMAT),
+		  val2valstr(curstat->mem.minflt,      format1, 6),
+		  val2valstr(curstat->mem.majflt,      format2, 6),
+		  val2valstr(curstat->mem.iosw,        format3, 4),
+		  val2memstr(curstat->mem.shtext*1024, format4, KBFORMAT),
+		  val2memstr(curstat->mem.vmem  *1024, format5, KBFORMAT),
+		  val2memstr(curstat->mem.rmem  *1024, format6, KBFORMAT),
+		  val2memstr(curstat->mem.vgrow *1024, format7, KBFORMAT),
+		  val2memstr(curstat->mem.rgrow *1024, format8, KBFORMAT),
 		  perc, curstat->gen.name);
 	}
 	else
@@ -360,17 +428,94 @@ showmemproc(struct pstat *curstat, double perc)
 		/*
 		** show process-info of exited process
 		*/
+	   if (kernpatch & PATCHACCT)
+	   {
+		printg(memproclinep,
+		  curstat->gen.pid,
+		  val2valstr(curstat->mem.minflt,      format1, 6),
+		  val2valstr(curstat->mem.majflt,      format2, 6),
+		  val2valstr(curstat->mem.iosw,        format3, 4),
+		  val2memstr(curstat->mem.vmem  *1024, format5, KBFORMAT),
+		  val2memstr(curstat->mem.rmem  *1024, format6, KBFORMAT),
+		  val2memstr(curstat->mem.vgrow *1024, format7, KBFORMAT),
+		  val2memstr(curstat->mem.rgrow *1024, format8, KBFORMAT),
+		  perc, curstat->gen.name);
+	   }
+	   else
+	   {
 		if (curstat->gen.pid == 0)
 			printg("    ? ");
 		else
 			printg("%5d ", curstat->gen.pid);
 
 		printg(memprocline,
-		  val2valstr(curstat->mem.minflt,     format1, 6),
-		  val2valstr(curstat->mem.majflt,     format2, 6),
-		  val2valstr(curstat->mem.iosw,       format3, 4),
+			  val2valstr(curstat->mem.minflt,     format1, 6),
+			  val2valstr(curstat->mem.majflt,     format2, 6),
+			  val2valstr(curstat->mem.iosw,       format3, 4),
+			  perc, curstat->gen.name);
+	   }
+	}
+}
+
+/*
+** print a line of disk-figures about one process
+*/
+void
+showdskproc(struct pstat *curstat, double perc)
+{
+	char	format1[16], format2[16], format3[16];
+	char	format4[16], format5[16], format6[16];
+	int	avgrsz, avgwsz;
+
+	avgrsz = curstat->dsk.rio ?
+	         curstat->dsk.rsz * 512LL / curstat->dsk.rio : 0;
+	avgwsz = curstat->dsk.wio ?
+	         curstat->dsk.wsz * 512LL / curstat->dsk.wio : 0;
+
+	if (curstat->gen.state != 'E')
+	{
+		/*
+		** show process-info of active process
+		*/
+		printg(dskproclina,
+		  curstat->gen.pid,
+		  val2valstr(curstat->dsk.rio,      	format1, 9),
+		  val2valstr(avgrsz,  			format2, 7),
+		  val2memstr(curstat->dsk.rsz * 512LL,	format3, KBFORMAT),
+		  val2valstr(curstat->dsk.wio,      	format4, 9),
+		  val2valstr(avgwsz,  			format5, 7),
+		  val2memstr(curstat->dsk.wsz * 512LL,	format6, KBFORMAT),
 		  perc, curstat->gen.name);
 	}
+	else
+	{
+		/*
+		** show process-info of exited process
+		*/
+	   if (kernpatch & PATCHACCT)
+	   {
+		printg(dskproclinep,
+		  curstat->gen.pid,
+		  val2valstr(curstat->dsk.rio,      	format1, 9),
+		  val2valstr(avgrsz,  			format2, 7),
+		  val2memstr(curstat->dsk.rsz * 512LL,	format3, KBFORMAT),
+		  val2valstr(curstat->dsk.wio,      	format4, 9),
+		  val2valstr(avgwsz,  			format5, 7),
+		  val2memstr(curstat->dsk.wsz * 512LL,	format6, KBFORMAT),
+		  perc, curstat->gen.name);
+	   }
+	   else
+	   {
+		if (curstat->gen.pid == 0)
+			printg("    ? ");
+		else
+			printg("%5d ", curstat->gen.pid);
+
+		printg(dskprocline,
+		  	val2valstr(curstat->dsk.rio, format1, 9),
+			perc, curstat->gen.name);
+	   }
+	}
 }
 
 /*
@@ -379,8 +524,18 @@ showmemproc(struct pstat *curstat, double perc)
 void
 shownetproc(struct pstat *curstat, double perc)
 {
-	char	format1[16], format2[16];
-	char	format3[16], format4[16];
+	char	format1[16], format2[16], format3[16], format4[16];
+	char	format5[16], format6[16], format7[16], format8[16];
+	int	avgtcpr, avgtcps, avgudpr, avgudps;
+
+	avgtcpr = curstat->net.tcprcv ?
+		  curstat->net.tcprsz / curstat->net.tcprcv : 0;
+	avgtcps = curstat->net.tcpsnd ?
+		  curstat->net.tcpssz / curstat->net.tcpsnd : 0;
+	avgudpr = curstat->net.udprcv ?
+		  curstat->net.udprsz / curstat->net.udprcv : 0;
+	avgudps = curstat->net.udpsnd ?
+		  curstat->net.udpssz / curstat->net.udpsnd : 0;
 
 	if (curstat->gen.state != 'E')
 	{
@@ -389,10 +544,16 @@ shownetproc(struct pstat *curstat, double perc)
 		*/
 		printg(netproclina,
 		  curstat->gen.pid,
-		  val2valstr(curstat->net.tcprcv,      format1, 7),
-		  val2valstr(curstat->net.tcpsnd,      format2, 7),
-		  val2valstr(curstat->net.udprcv,      format3, 7),
-		  val2valstr(curstat->net.udpsnd,      format4, 7),
+		  val2valstr(curstat->net.tcprcv,      format1,  6),
+		  val2valstr(avgtcpr,                  format2,  4),
+		  val2valstr(curstat->net.tcpsnd,      format3,  6),
+		  val2valstr(avgtcps,                  format4,  4),
+		  val2valstr(curstat->net.udprcv,      format5,  6),
+		  val2valstr(avgudpr,                  format6,  4),
+		  val2valstr(curstat->net.udpsnd,      format7,  6),
+		  val2valstr(avgudps,                  format8,  4),
+		  curstat->net.rawrcv,
+		  curstat->net.rawsnd,
 		  perc, curstat->gen.name);
 	}
 	else
@@ -400,12 +561,31 @@ shownetproc(struct pstat *curstat, double perc)
 		/*
 		** show process-info of exited process
 		*/
+	   if (kernpatch & PATCHACCT)
+	   {
+		printg(netproclinep,
+		  curstat->gen.pid,
+		  val2valstr(curstat->net.tcprcv,      format1,  6),
+		  val2valstr(avgtcpr,                  format2,  4),
+		  val2valstr(curstat->net.tcpsnd,      format3,  6),
+		  val2valstr(avgtcps,                  format4,  4),
+		  val2valstr(curstat->net.udprcv,      format5,  6),
+		  val2valstr(avgudpr,                  format6,  4),
+		  val2valstr(curstat->net.udpsnd,      format7,  6),
+		  val2valstr(avgudps,                  format8,  4),
+		  curstat->net.rawrcv,
+		  curstat->net.rawsnd,
+		  perc, curstat->gen.name);
+	   }
+	   else
+	   {
 		if (curstat->gen.pid == 0)
 			printg("    ? ");
 		else
 			printg("%5d ", curstat->gen.pid);
 
 		printg(netprocline, perc, curstat->gen.name);
+	   }
 	}
 }
 
@@ -413,7 +593,7 @@ shownetproc(struct pstat *curstat, double perc)
 ** print a line of (more or less) static info about one process
 */
 void
-showstaproc(struct pstat *curstat, double perc)
+showvarproc(struct pstat *curstat, double perc)
 {
 	char		format1[16], format2[16], format3[16];
 	struct passwd 	*pwd;
@@ -441,7 +621,7 @@ showstaproc(struct pstat *curstat, double perc)
 		/*
 		** show process-info of active process
 		*/
-		printg(staproclina,
+		printg(varproclina,
 		  	curstat->gen.pid,
 		  	username,
 		  	groupname,
@@ -469,7 +649,7 @@ showstaproc(struct pstat *curstat, double perc)
 				 curstat->gen.excode     & 0x7f :
 		  		(curstat->gen.excode>>8) & 0xff );
 
-		printg(staprocline,
+		printg(varprocline,
 		  	username,
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
@@ -516,6 +696,78 @@ showcmdproc(struct pstat *curstat, double perc)
 	}
 }
 
+/*
+** print a line of scheduling info about one process
+*/
+#define SCHED_NORMAL            0
+#define SCHED_FIFO              1
+#define SCHED_RR                2
+
+void
+showschproc(struct pstat *curstat, double perc)
+{
+	char	format1[16];
+	char	*pol;
+
+	switch (curstat->cpu.policy)
+	{
+	   case SCHED_NORMAL:
+		pol = "normal";
+		break;
+	   case SCHED_RR:
+		pol = "roundr";
+		break;
+	   case SCHED_FIFO:
+		pol = "fifo";
+		break;
+	   default:
+		pol = "?";
+	}
+
+	/*
+	** show process-info
+	*/
+	if (curstat->gen.state != 'E')
+	{
+		/*
+		** show process-info of active process
+		*/
+		printg(schproclina,
+		  	curstat->gen.pid,
+		  	curstat->gen.tgid,
+		  	curstat->gen.nthr,
+			curstat->gen.pid == curstat->gen.tgid ? 'L' : 'M',
+		  	pol,
+		  	curstat->cpu.nice,
+		  	curstat->cpu.prio,
+		  	curstat->cpu.rtprio,
+		  	curstat->cpu.curcpu,
+			curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
+		  	curstat->gen.state,
+		  	perc, curstat->gen.name);
+	}
+	else
+	{
+		/*
+		** show process-info of exited process
+		*/
+		if (curstat->gen.pid == 0)
+			printg("    ?");
+		else
+			printg("%5d", curstat->gen.pid);
+
+		sprintf(format1, "%c%c%c%3ld",
+			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+		  	curstat->gen.excode & 0xff        ? 'S' : 'E',
+		  	curstat->gen.excode & 0x80        ? 'C' : '-',
+		  	curstat->gen.excode & 0xff        ? 
+				 curstat->gen.excode     & 0x7f :
+		  		(curstat->gen.excode>>8) & 0xff );
+
+		printg(schprocline,
+			format1, perc, curstat->gen.name);
+	}
+}
 
 /*
 ** print the header for the process-list
@@ -526,7 +778,7 @@ priphead(int curlist, int totlist, char *sortorder, char showtype)
 	switch (showtype)
 	{
 	   case 'g':
-		if (!kernpatch)
+		if ( !(kernpatch & PATCHSTAT) )
 			printg(genprochdr1, sortorder, curlist, totlist);
 		else
 			printg(genprochdr2, sortorder, curlist, totlist);
@@ -536,17 +788,25 @@ priphead(int curlist, int totlist, char *sortorder, char showtype)
 		printg(memprochdr, sortorder, curlist, totlist);
 		break;
 
+	   case 'd':
+		printg(dskprochdr, sortorder, curlist, totlist);
+		break;
+
 	   case 'n':
 		printg(netprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 's':
-		printg(staprochdr, sortorder, curlist, totlist);
+	   case 'v':
+		printg(varprochdr, sortorder, curlist, totlist);
 		break;
 
 	   case 'c':
 		printg(cmdprochdr, sortorder, curlist, totlist);
 		break;
+
+	   case 's':
+		printg(schprochdr, sortorder, curlist, totlist);
+		break;
 	}
 }
 
@@ -584,7 +844,7 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 			if (sb->availcpu)
 				perc = (double)(curstat->cpu.stime +
 				                curstat->cpu.utime  ) *
-						100.0 / sb->availcpu;
+					        100.0 / sb->availcpu;
 			else
 				perc = 0.0;
 			break;
@@ -599,8 +859,8 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 
 		   case 'D':
 			if (sb->availdsk)
-				perc = (double)(curstat->dsk.ior +
-				                curstat->dsk.iow  ) *
+				perc = (double)(curstat->dsk.rio +
+				                curstat->dsk.wio  ) *
 			        		100.0 / sb->availdsk;
 			else
 				perc = 0.0;
@@ -611,7 +871,9 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 				perc = (double)(curstat->net.tcpsnd +
 						curstat->net.tcprcv +
 						curstat->net.udpsnd +
-						curstat->net.udprcv  ) *
+						curstat->net.udprcv +
+						curstat->net.rawsnd +
+						curstat->net.rawrcv ) *
 						100.0 / sb->availnet;
 			else
 				perc = 0.0;
@@ -664,17 +926,25 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 			showmemproc(curstat, perc);
 			break;
 
+		   case 'd':
+			showdskproc(curstat, perc);
+			break;
+
 		   case 'n':
 			shownetproc(curstat, perc);
 			break;
 
-		   case 's':
-			showstaproc(curstat, perc);
+		   case 'v':
+			showvarproc(curstat, perc);
 			break;
 
 		   case 'c':
 			showcmdproc(curstat, perc);
 			break;
+
+		   case 's':
+			showschproc(curstat, perc);
+			break;
 		}
 
 		curline++;
@@ -708,26 +978,28 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 	if (cputot == 0)
 		cputot = 1;		/* avoid divide-by-zero */
 
+	percputot = cputot / sstat->cpu.nrcpu;
+
 	if (osrel == 2 && osvers >= 6)
 	{
-		printg("CPU | system %3.0f%% | user    %3.0f%% | "
+		printg("CPU | sys %6.0f%% | user %6.0f%% | "
 		       "irq  %6.0f%% | idle %6.0f%% | wait %6.0f%% |\n",
-			(double)(sstat->cpu.all.stime * 100.0) / cputot,
+			(double) (sstat->cpu.all.stime * 100.0) / percputot,
 			(double)((sstat->cpu.all.utime + sstat->cpu.all.ntime)
-			                              * 100.0) / cputot,
+			                               * 100.0) / percputot,
 			(double)((sstat->cpu.all.Itime + sstat->cpu.all.Stime)
-			                              * 100.0) / cputot,
-			(double)(sstat->cpu.all.itime * 100.0) / cputot,
-			(double)(sstat->cpu.all.wtime * 100.0) / cputot);
+			                               * 100.0) / percputot,
+			(double) (sstat->cpu.all.itime * 100.0) / percputot,
+			(double) (sstat->cpu.all.wtime * 100.0) / percputot);
 	}
 	else
 	{
-		printg("CPU | system %3.0f%% | user    %3.0f%% | "
+		printg("CPU | sys %6.0f%% | user %6.0f%% | "
 		       "nice %6.0f%% | idle %6.0f%% | cs %s |\n",
-			(double)(sstat->cpu.all.stime * 100.0) / cputot,
-			(double)(sstat->cpu.all.utime * 100.0) / cputot,
-			(double)(sstat->cpu.all.ntime * 100.0) / cputot,
-			(double)(sstat->cpu.all.itime * 100.0) / cputot,
+			(double)(sstat->cpu.all.stime * 100.0) / percputot,
+			(double)(sstat->cpu.all.utime * 100.0) / percputot,
+			(double)(sstat->cpu.all.ntime * 100.0) / percputot,
+			(double)(sstat->cpu.all.itime * 100.0) / percputot,
 			val2valstr(sstat->cpu.csw, format1, 9));
 	}
 
@@ -755,26 +1027,27 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 
 			if (osrel == 2 && osvers >= 6)
 			{
-				printg("CPU | system %3.0f%% | "
-				       "user    %3.0f%% | irq  %6.0f%% | "
-				       "idlewt %4.0f%% |       cpu%03d |\n",
-					(double)(sstat->cpu.cpu[i].stime 
-							* 100.0) / cputot,
+				printg("cpu | sys %6.0f%% | "
+				       "user %6.0f%% | irq  %6.0f%% | "
+				       "idle %6.0f%% | cpu%03d w%3.0f%% |\n",
+					(double) (sstat->cpu.cpu[i].stime 
+							* 100.0) / percputot,
 					(double)((sstat->cpu.cpu[i].utime +
 					          sstat->cpu.cpu[i].ntime)
-			                              	* 100.0) / cputot,
+			                              	* 100.0) / percputot,
 					(double)((sstat->cpu.cpu[i].Itime +
 					          sstat->cpu.cpu[i].Stime)
-			                              	* 100.0) / cputot,
-					(double)((sstat->cpu.cpu[i].itime +
-					          sstat->cpu.cpu[i].wtime)
-							* 100.0) / cputot,
-					i);
+			                              	* 100.0) / percputot,
+					(double) (sstat->cpu.cpu[i].itime
+							* 100.0) / percputot,
+					sstat->cpu.cpu[i].cpunr,
+					(double) (sstat->cpu.cpu[i].wtime
+							* 100.0) / percputot);
 			}
 			else
 			{
-				printg("CPU | system %3.0f%% | "
-				       "user    %3.0f%% | nice %6.0f%% | "
+				printg("cpu | sys %6.0f%% | "
+				       "user %6.0f%% | nice %6.0f%% | "
 				       "idle %6.0f%% |       cpu%03d |\n",
 			   		(double)(sstat->cpu.cpu[i].stime
 							*100.0) /percputot,
@@ -784,7 +1057,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 							*100.0) /percputot,
 			   		(double)(sstat->cpu.cpu[i].itime
 							*100.0) /percputot,
-			   		i);	
+					sstat->cpu.cpu[i].cpunr);
 			}
 			curline++;
 			lin++;
@@ -806,12 +1079,13 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 	/*
 	** SWAP statistics
 	*/
-	printg("SWP | tot  %s | free  %s |              | "
+	printg("SWP | tot  %s | free  %s | vmcom %s | "
 	       "swin %s | swout %s |\n",
-		val2memstr(sstat->mem.totswap  * pgsz, format1, MBFORMAT),
-		val2memstr(sstat->mem.freeswap * pgsz, format2, MBFORMAT),
-		val2valstr(sstat->mem.swins,         format3, 7),
-		val2valstr(sstat->mem.swouts,        format4, 6));
+		val2memstr(sstat->mem.totswap   * pgsz, format1, MBFORMAT),
+		val2memstr(sstat->mem.freeswap  * pgsz, format2, MBFORMAT),
+		val2memstr(sstat->mem.committed * pgsz, format3, MBFORMAT),
+		val2valstr(sstat->mem.swins,            format4, 7),
+		val2valstr(sstat->mem.swouts,           format5, 6));
 
 	curline++;
 
@@ -910,10 +1184,10 @@ compcpu(const void *a, const void *b)
 int
 compdsk(const void *a, const void *b)
 {
-	register count_t adsk = ((struct pstat *)a)->dsk.ior +
-			        ((struct pstat *)a)->dsk.iow;
-	register count_t bdsk = ((struct pstat *)b)->dsk.ior +
-			        ((struct pstat *)b)->dsk.iow;
+	register count_t adsk = ((struct pstat *)a)->dsk.rio +
+			        ((struct pstat *)a)->dsk.wio;
+	register count_t bdsk = ((struct pstat *)b)->dsk.rio +
+			        ((struct pstat *)b)->dsk.wio;
 
 	if (adsk < bdsk) return  1;
 	if (adsk > bdsk) return -1;
@@ -937,11 +1211,15 @@ compnet(const void *a, const void *b)
 	register count_t anet = ((struct pstat *)a)->net.tcpsnd +
 			        ((struct pstat *)a)->net.tcprcv +
 			        ((struct pstat *)a)->net.udpsnd +
-			        ((struct pstat *)a)->net.udprcv  ;
+			        ((struct pstat *)a)->net.udprcv +
+			        ((struct pstat *)a)->net.rawsnd +
+			        ((struct pstat *)a)->net.rawrcv  ;
 	register count_t bnet = ((struct pstat *)b)->net.tcpsnd +
 			        ((struct pstat *)b)->net.tcprcv +
 			        ((struct pstat *)b)->net.udpsnd +
-			        ((struct pstat *)b)->net.udprcv  ;
+			        ((struct pstat *)b)->net.udprcv +
+			        ((struct pstat *)b)->net.rawsnd +
+			        ((struct pstat *)b)->net.rawrcv  ;
 
 	if (anet < bnet) return  1;
 	if (anet > bnet) return -1;
@@ -951,8 +1229,10 @@ compnet(const void *a, const void *b)
 int
 cpucompar(const void *a, const void *b)
 {
-	register count_t aidle = ((struct percpu *)a)->itime;
-	register count_t bidle = ((struct percpu *)b)->itime;
+	register count_t aidle = ((struct percpu *)a)->itime +
+	                         ((struct percpu *)a)->wtime;
+	register count_t bidle = ((struct percpu *)b)->itime +
+	                         ((struct percpu *)b)->wtime;
 
 	if (aidle < bidle) return -1;
 	if (aidle > bidle) return  1;
